{
  User interface to sampling software
  Jan Wassenberg 2000
}

{$M $0400, $0000, $0000} {Allocate 1 KB stack space}

program User_Interface; {v7.6.2, 03/12/2000}

uses CRT;

const
  Choices : array[0..2] of string[11] = ('Kapazitt  ', 'Temperatur ', 'Luftfeuchte');
  MinVal  : array[0..2] of Double = (0.0     , 1.7E+308, 10.0);
  MaxVal  : array[0..2] of Double = (1.7E+308, 1.7E+308, 90.0);

var
  OldExitProc : Pointer;

  i : Word;
  Choice : Byte;

  CableAttached : Boolean;
  NeedRefresh : Boolean;
  SampleActive, Changed : Boolean;

  Count : Word;
  Value, Min, Max : Double;
  Sum, Avg : Extended;

  ScaleMin, ScaleMax, ScaleAvg : LongInt;
  MinPos, MaxPos, AvgPos : Word;

procedure ExitHandler; far;
begin
  ExitProc := OldExitProc;

  {Set text mode}
  asm
    MOV     AX, $0003
    INT     $10
  end;

  if SampleActive then WriteLn('Messung wurde abgebrochen');
  WriteLn('Programm wird beendet...');
end;

{$L SAMPLE}
procedure init; external;
function  check_com : Boolean; external;
procedure set_sample_type (SampleType : Word); external;
function  sample : Double; external;

{$L GFX}
procedure h_line (X1, X2, Y, Color : Word); external;
procedure v_line (X, Y1, Y2, Color : Word); external;

{$L MOUSE}
function  reset_mouse : Word; external;
procedure show_mouse; external;
procedure hide_mouse; external;
function  mouse_x : Integer; external;
function  mouse_in_rect (X1, Y1, X2, Y2 : Word) : Boolean; external;
function  get_left_clicks : Word; external;

{Draw a display line}
procedure DrawLine (Value : Double; Color : Byte; var OldPos : Word);
var
  Pos : Word;
begin
  Pos := Round((Value - ScaleAvg) / (ScaleMax - ScaleAvg) * 240 + 320);
  if (Pos <> OldPos) then begin
    if (Pos > 0) and (Pos < 639) then begin
      v_line(Pos   , 230, 250, Color);
      v_line(OldPos, 230, 250, 0);
      OldPos := Pos;
    end {of if}
    else begin
      SampleActive := False;
      NeedRefresh  := True;
    end; {of else}
  end; {of if}
end;

{Returns Value in " iii,ff uu" format}
function FormatStr (Value : Double) : string;
const
  Units    : array[1..2] of string[2] = ('C', '% ');
  CapUnits : array[0..4] of string[2] = ('pF', 'nF', 'F', 'mF', 'F ');
var
  Factor : Byte;
  IntStr, FracStr, UnitStr : string;
begin
  if Choice = 0 then begin
    Factor := 0;
    while Value > 1000.0 do begin
      Value := Value / 1000.0;
      Inc(Factor);
    end;
    UnitStr := CapUnits[Factor];
  end
  else UnitStr := Units[Choice];
  Value := Round(Value * 1E2) / 1E2; {Strip all but 2 decimal places}
  Str(Trunc(Value), IntStr);
  Str(Round(Frac(Value) * 1E2), FracStr);
  if Length(FracStr) = 1 then FracStr := '0' + FracStr; {x.x => x.0x}
  FormatStr := IntStr + ',' + FracStr + ' ' + UnitStr;
end;

begin
  OldExitProc := ExitProc;
  ExitProc := @ExitHandler;

  Init; {Stabilize cpuid and calculate CPU clock and I/O speed}

  {Video init}
  asm
    MOV     AX, $0012
    INT     $10
  end;
  DirectVideo := False; {Use BIOS calls for Write, WriteLn}
  WindMax := $1D4F; {640x480 with 8x16 character cells => 80x30 characters}

  Write(' C/T/F Messprogramm ');
  Write('                                 Jan Wassenberg                                 ');
  GotoXY(7, 30);
  Write('<Escape> beendet das Programm (grere Kondensatoren vorher abklemmen)');

  {Mouse init}
  reset_mouse; {Initialize mouse driver}
  show_mouse;

  NeedRefresh := True; {Initial screen draw}

  repeat
    {Check if cable connected to computer}
    if CableAttached <> check_com then begin
      CableAttached := check_com;
      NeedRefresh := True;
    end; {of if}

    {Handle user input (mouse clicks)}
    if get_left_clicks > 0 then begin
      if mouse_in_rect(168, 112, 463, 127) and not SampleActive then begin {New sample type}
	Choice := (mouse_x - 168) div 103;
	set_sample_type(Choice);
	NeedRefresh := True;
      end {of if}
      else if mouse_in_rect(256, 144, 375, 159) then begin {Start/Stop}
	SampleActive := not SampleActive;
	if SampleActive then begin
	  Count := 0; Sum := 0.0;
	  Min := 1.7E+308; Max := 0.0;
	  GotoXY(1, 15);
	  for i := 1 to 7 do Write(' ':80);
          Value := Sample;
	  if (Value >= MinVal[Choice]) and (Value <= MaxVal[Choice]) then begin
            {Display status}
            GotoXY(1, 3);
            WriteLn(' ':80);
            WriteLn('Messung luft...');

            {Display scale}
            h_line(0  , 639, 250, $0707);
            for i := 0 to 592 do begin
	      if i mod 48  = 0 then v_line(i + 32, 250, 253, $07);
              if i mod 240 = 0 then v_line(i + 80, 253, 256, $07);
            end; {of for}

            {Calculate & display scale values}
            ScaleMin := Round(0.098 * Value) * 10; {- 2%, rounded to 10}
	    ScaleMax := Round(0.102 * Value) * 10; {+ 2%, rounded to 10}
	    if ScaleMin = ScaleMax then begin
              Dec(ScaleMin, 10);
              Inc(ScaleMax, 10);
            end; {of if}
            ScaleAvg := (ScaleMin + ScaleMax) shr 1;
            GotoXY(1, 17);
	    Write(FormatStr(ScaleMin):15, FormatStr(ScaleAvg):30, FormatStr(ScaleMax):30);
	  end {of if}
	  else SampleActive := False;
	end; {of if}
        NeedRefresh := True;
      end; {of else if}
    end; {of if}

    {Update display}
    if NeedRefresh then begin
      hide_mouse;
      GotoXY(1, 5);
      if CableAttached then begin
	if SampleActive then begin
	  WriteLn('                                Messung luft...                       ');
	  for i := 1 to 3 do Write(' ':80);
	end {of if}
	else begin
	  WriteLn('               Stecken Sie das Sensormodul in das Messkabel ein;       ');
	  WriteLn('              danach whlen Sie per Mausclick die Art der Messung:');
	  GotoXY(20, 8);
	  for i := 0 to 2 do begin
	    if i = Choice then TextAttr := 15 else TextAttr := 7;
	    Write('  ', Choices[i]);
	  end; {of for}
	end; {of else}
	GotoXY(33, 10);
	TextAttr := 15;
	if SampleActive then
	  Write('Messung anhalten')
	else
	  Write('Messung starten ');
	TextAttr := 7;
      end
      else begin
	Write    ('           Stecken Sie das Messkabel in eine serielle Schnittstelle ein');
	for i := 1 to 5 do Write(' ':80);
      end; {of else}
      show_mouse;
      NeedRefresh := False;
    end; {of if}

    {Run sample}
    if SampleActive then begin
      {Get current value}
      Value := Sample;
      if (Value >= MinVal[Choice]) and (Value <= MaxVal[Choice]) then begin
	{Calculate average}
	Inc(Count);
	Sum := Sum + Value;
	Avg := Sum / Count; {Average}
	{Update Min/Max}
	Changed := False;
	if Value < Min then begin
	  Min := Value;
	  Changed := True;
	end;
	if Value > Max then begin
	  Max := Value; {Maximum}
	  Changed := True;
	end;
	hide_mouse;
	DrawLine(Avg, 9, AvgPos);
	GotoXY(28, 19);
	Write('Mittelwert:', FormatStr(Avg):10);
	if Changed then begin
	  DrawLine(Min, 12, MinPos);
	  DrawLine(Max, 12, MaxPos);
	  GotoXY(1, 19);
	  Write('Minimum:', FormatStr(Min):10);
	  GotoXY(63, 19);
	  Write('Maximum:', FormatStr(Max):10);
	end; {of if}
	GotoXY(27, 21);
	WriteLn(Count:5, ' Messungen durchgefhrt');
	show_mouse;
	Delay(20);
      end {of if}
      else begin
	SampleActive := False;
	NeedRefresh  := True;
      end; {of else}
    end; {of if}
  until KeyPressed and (ReadKey = #27);
end.